'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toMakeTheme = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = createMakeTheme;
exports.makeScopedTheme = makeScopedTheme;

var _lodash = require('lodash.merge');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Convert a theme or a scopedTheme into a makeTheme.
 * @param theme
 *
 *
 * @example
 * const darkTheme = originalMakeTheme({
 *    '$body-bg': 'dark',
 *    '$body-color': 'white',
 *    '$btn-primary-bg': 'red',
 * });
 *
 * const darkMakeTheme = toMakeTheme(darkTheme);
 *
 * Now you can create a new theme starting from the personalized theme as default theme rather than the default original theme.
 * This allows others to build from your theme as a starting point and makes it a lot easier to share and make new themes.
 *
 * const anotherDarkTheme = darkMakeTheme({
 *     '$body-bg': '#121212'
 * });
 *
 * The method also takes care of scopedTheme so that you don't have to worry about a thing.
 *
 * const loginFormTheme = makeScopedTheme('loginForm', {
 *    '$body-bg': 'dark',
 *    '$body-color': 'white',
 *    '$btn-primary-bg': 'red',
 * })
 *
 * const loginFormMakeTheme = toMakeTheme(loginFormTheme);
 *
 * const anotherLoginFormTheme = loginFormMakeTheme({
 *     loginForm: {
 *        '$body-bg': '#121212'
 *     }
 * });
 */
var toMakeTheme = exports.toMakeTheme = function toMakeTheme(theme) {
  return function (userTheme) {
    // eslint-disable-line arrow-body-style
    return (0, _lodash2.default)(theme, userTheme);
  };
};

function makeThemeFromList(list, theme) {
  var all = [].concat(list);
  var t = theme;
  var mt = void 0;
  while (mt = all.pop()) {
    // eslint-disable-line
    t = mt(t);
  }
  return t;
}

/**
 * Create a makeTheme using a list of makeThemes.
 * @param list
 * @returns {function(*=): *}
 */
function createMakeTheme(list) {
  return function (theme) {
    return makeThemeFromList(list, theme);
  };
}

/**
* Creates a scoped makeTheme. Used for creating components or modules with their own variables.
* @param scopeName string
* @param userTheme
*
**/

function makeScopedTheme(scopeName) {
  var userTheme = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _defineProperty({}, scopeName, {});

  if (scopeName === undefined || typeof scopeName !== 'string') return console.warn('You may have forgotten to set the scope name in the makeScopedTheme function.');
  var newTheme = _defineProperty({}, scopeName, {});
  var v = newTheme[scopeName];
  var u = userTheme[scopeName] || {};

  Object.keys(userTheme).forEach(function (variable) {
    v[variable] = u[variable] || userTheme[variable];
  });

  return _defineProperty({}, scopeName, _extends({}, userTheme, v));
}